"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HyperswarmTransport = void 0;
const crypto_1 = require("crypto");
const hyperswarm_1 = __importDefault(require("hyperswarm"));
const ataraxia_transport_1 = require("ataraxia-transport");
const ataraxia_transport_streams_1 = require("ataraxia-transport-streams");
/**
 * Hyperswarm based transport. This transport will connect to
 * [Hyperswarm](https://github.com/hyperswarm/hyperswarm) and attempt to find
 * peers that are published under a specific topic. When peers are found they
 * will be connected to, a secure channel established and authentication will
 * occur.
 *
 * ```javascript
 * import { Network, AnonymousAuth } from 'ataraxia';
 * import { HyperswarmTransport } from 'ataraxia-hyperswarm';
 *
 * // Setup a network over Hyperswarm
 * const net = new Network({
 *   name: 'name-of-your-app-or-network',
 *   transports: [
 *     new HyperswarmTransport({
 *       // Topic used to find peers
 *       topic: 'Unique Topic',
 *       // Setup anonymous authentication
 *       authentication: [
 *         new AnonymousAuth()
 *       ]
 *     })
 *   ]
 * });
 *
 * // Join the network
 * await net.join();
 * ```
 */
class HyperswarmTransport extends ataraxia_transport_1.AbstractTransport {
    constructor(options) {
        super('hyperswarm');
        this.options = options;
    }
    async start(options) {
        const started = await super.start(options);
        if (!started)
            return false;
        this.swarm = (0, hyperswarm_1.default)({
            // Assume that endpoints are short-lived
            ephemeral: options.endpoint ? true : undefined
        });
        this.topic = (0, crypto_1.createHash)('sha256')
            .update(this.options.topic)
            .digest();
        this.debug('Joining the topic', this.topic.toString('hex'));
        this.swarm.on('connection', (socket, info) => {
            this.debug('Connecting to a peer, client=', info.client);
            this.addPeer(new HyperswarmPeer(this.transportOptions, this.options.authentication, socket, info.client));
        });
        const swarm = this.swarm;
        const topic = this.topic;
        await new Promise(resolve => {
            /*
             * Join the topic automatically setting announce and lookup.
             *
             * Endpoints join the topic without announcing they can be connected
             * to which makes them only find existing peers.
             *
             * If not an endpoint this will announce its availability and also
             * search for existing peers.
             */
            swarm.join(topic, {
                announce: !options.endpoint,
                lookup: true
            }, () => resolve(undefined));
        });
        this.debug('Swarm has been joined');
        return true;
    }
    async stop() {
        const stopped = await super.stop();
        if (!stopped)
            return false;
        const swarm = this.swarm;
        const topic = this.topic;
        if (!swarm || !topic)
            return false;
        await new Promise(resolve => {
            swarm.leave(topic, () => resolve(undefined));
        });
        return true;
    }
}
exports.HyperswarmTransport = HyperswarmTransport;
class HyperswarmPeer extends ataraxia_transport_streams_1.EncryptedStreamingPeer {
    constructor(transportOptions, authProviders, socket, client) {
        super(transportOptions, authProviders);
        this.setStream(socket, client);
    }
}
//# sourceMappingURL=HyperswarmTransport.js.map